﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;
using Niche.GroupInspector.Domain;
using Niche.GroupInspector.Presentation;

namespace GroupInspector
{
    public partial class GroupInspectorForm : Form, IGroupInspectorView
    {
        /// <summary>
        /// Gets or sets a reference to our Presenter
        /// </summary>
        public IGroupInspectorPresenter Presenter
        {
            get;
            set;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GroupInspectorForm"/> class.
        /// </summary>
        public GroupInspectorForm()
        {
            InitializeComponent();
            showVersion.Text = Assembly.GetEntryAssembly().GetName().Version.ToString();
        }

        /// <summary>
        /// Set the Application Systems available for selection by the User
        /// </summary>
        /// <param name="systems">Sequence of available systems</param>
        public void SetAvailableSystems(IEnumerable<ApplicationSystem> systems)
        {
            var systemList = new List<ApplicationSystem>(systems);
            editSystem.ComboBox.DataSource = systemList;
            editSystem.ComboBox.DisplayMember = "Name";
        }

        /// <summary>
        /// Display a specific access group
        /// </summary>
        /// Should add the group to the display, retaining any existing display
        /// <param name="groups">Group to display</param>
        /// <param name="users">Users found in group</param>
        public void DisplayAccessGroups(IEnumerable<AccessGroup> groups)
        {
            listAccess.Groups.Clear();
            listAccess.Items.Clear();

            foreach (var g in groups)
            {
                var listGroup = listAccess.Groups.Add(g.Name, g.DisplayName);
                if (g.Users != null)
                {
                    foreach (var u in g.Users)
                    {
                        AddUser(listGroup, u);
                    }
                }
            }
        }

        /// <summary>
        /// Display an error message to the user
        /// </summary>
        /// <param name="errorMessage">Message to display</param>
        public void DisplayError(string errorMessage, string toolTip)
        {
            var haveMessage = !string.IsNullOrEmpty(errorMessage);
            paneErrors.Visible = haveMessage;
            showErrorMessage.Text = errorMessage;
            showTooltips.SetToolTip(showErrorMessage, toolTip);
        }

        /// <summary>
        /// Display progress update to the user
        /// </summary>
        /// <param name="progressMessage">Message to display</param>
        /// <param name="completed">Number of completed steps</param>
        /// <param name="total">Number of steps in total</param>
        public void DisplayProgress(string progressMessage, int completed, int total)
        {
            showProgressMessage.Text = progressMessage;
            showProgress.Maximum = total;
            showProgress.Value = completed;
            paneProgress.Visible = completed < total;
            Update();
        }

        /// <summary>
        /// Configure display to a particular style
        /// </summary>
        /// <param name="style">Style of view to configure</param>
        public void SetViewStyle(ViewStyle style)
        {
            switch (style)
            {
                case ViewStyle.Details:
                    listAccess.SmallImageList = listDetailImages;
                    listAccess.View = View.Details;
                    break;

                case ViewStyle.Icons:
                    listAccess.LargeImageList = listIconImages;
                    listAccess.View = View.LargeIcon;
                    break;

                case ViewStyle.Tiles:
                    listAccess.LargeImageList = listTileImages;
                    listAccess.View = View.Tile;
                    break;

                case ViewStyle.List:
                    listAccess.SmallImageList = listDetailImages;
                    listAccess.View = View.SmallIcon;
                    break;

                default:
                    listAccess.View = View.Tile;
                    break;
            }

            menuViewTiles.Checked = style == ViewStyle.Tiles;
            menuViewIcons.Checked = style == ViewStyle.Icons;
            menuViewDetails.Checked = style == ViewStyle.Details;
        }

        public void SetViewOrder(ViewOrder order)
        {
            menuSortByFamilyName.Checked = order == ViewOrder.FamilyName;
            menuSortByName.Checked = order == ViewOrder.None;
            menuSortByUserName.Checked = order == ViewOrder.UserName;

            Presenter.SetOrder(order);
        }

        private void AddUser(ListViewGroup listGroup, User user)
        {
            var item = listAccess.Items.Add(user.Name, user.Name, "user");
            item.SubItems.Add(user.UserName);
            item.Group = listGroup;
        }

        private void editSystem_SelectedValueChanged(object sender, EventArgs e)
        {
            var system = editSystem.SelectedItem as ApplicationSystem;
            Presenter.SelectApplicationSystem(system);
        }

        private void labelHome_Click(object sender, EventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start("http://www.nichesoftware.co.nz");
            }
            catch
            {
                // Ignore any problems here
            }
        }

        private void listAccess_DrawItem(object sender, DrawListViewItemEventArgs e)
        {
            e.DrawDefault = true;
        }

        private void toolStripStatusLabel1_Click(object sender, EventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start("http://www.icon-king.com/projects/nuvola/");
            }
            catch
            {
                // Ignore any problems here
            }
        }

        private void GroupInspectorForm_Load(object sender, EventArgs e)
        {
            paneTools.LayoutStyle = ToolStripLayoutStyle.Table;
            var layout = paneTools.LayoutSettings as TableLayoutSettings;

            layout.RowStyles.Add(new RowStyle(SizeType.AutoSize));

            layout.ColumnStyles.Add(new ColumnStyle(SizeType.AutoSize));
            layout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 100));
            layout.ColumnStyles.Add(new ColumnStyle(SizeType.AutoSize));
            layout.ColumnStyles.Add(new ColumnStyle(SizeType.AutoSize));

            layout.RowCount = layout.RowStyles.Count;
            layout.ColumnCount = layout.ColumnStyles.Count;

            labelSystem.Anchor = AnchorStyles.Left;
            editSystem.Anchor = AnchorStyles.Left | AnchorStyles.Right;

            buttonView.Anchor = AnchorStyles.None;
            buttonSort.Anchor = AnchorStyles.None;
        }

        private void menuViewTiles_Click(object sender, EventArgs e)
        {
            Presenter.SetViewStyle(ViewStyle.Tiles);
        }

        private void menuViewIcons_Click(object sender, EventArgs e)
        {
            Presenter.SetViewStyle(ViewStyle.Icons);
        }

        private void menuViewDetails_Click(object sender, EventArgs e)
        {
            Presenter.SetViewStyle(ViewStyle.Details);
        }

        private void byFamilyNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetViewOrder(ViewOrder.FamilyName);
        }

        private void byNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetViewOrder(ViewOrder.Name);
        }

        private void byUserNameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SetViewOrder(ViewOrder.UserName);
        }

        private void menuViewList_Click(object sender, EventArgs e)
        {
            Presenter.SetViewStyle(ViewStyle.List);
        }
    }
}
