﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.DirectoryServices;
using System.DirectoryServices.AccountManagement;
using System.Globalization;
using System.Linq;
using Niche.GroupInspector.Domain;
using Niche.Shared;

namespace Niche.GroupInspector.Presentation
{
    public class ActiveDirectoryGroupModel : IGroupInspectorModel
    {
        /// <summary>
        /// Gets a reference to the provider we use to supply Application Systems
        /// </summary>
        public IApplicationSystemProvider Provider
        {
            get;
            private set;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ActiveDirectoryGroupModel"/> class.
        /// </summary>
        /// <param name="provider">Provider to use to supply Application Systems.</param>
        public ActiveDirectoryGroupModel(IApplicationSystemProvider provider)
        {
            provider.RequireNotNull("provider");
            Provider = provider;
        }

        /// <summary>
        /// Load our sequence of available application systems
        /// </summary>
        /// <returns>Sequence of application systems.</returns>
        public IEnumerable<ApplicationSystem> LoadApplicationSystems()
        {
            return Provider.ProvideApplicationSystems();
        }

        /// <summary>
        /// Get all the users who are a part of the specified access group
        /// </summary>
        /// <param name="accessGroup">Access group for which users are required</param>
        /// <returns>Sequence of found users (possibly empty)</returns>
        public IEnumerable<User> GetUsersForAccessGroup(AccessGroup accessGroup)
        {
            // Find the group itself
            var context = new PrincipalContext(ContextType.Domain);
            var groupSpecification
                = new GroupPrincipal(context)
                      {
                          Name = accessGroup.ActiveDirectoryGroupName
                      };
            var searcher = new PrincipalSearcher(groupSpecification);
            var group = (GroupPrincipal)searcher.FindOne();

            if (group == null)
            {
                string message
                    = string.Format(
                        CultureInfo.CurrentCulture,
                        "Failed to find Active Directory group {0}",
                        accessGroup.Name);
                throw new InvalidOperationException(message);
            }

            var members = group.GetMembers(true);
            var users
                = members.OfType<UserPrincipal>()
                    .Select(
                    u => new User()
                             {
                                 Name = u.DisplayName,
                                 Description = u.Description,
                                 DistinguishedName = u.DistinguishedName,
                                 UserName = u.SamAccountName,
                                 Surname = u.Surname
                             })
                    .ToList();

            return users;
        }
    }
}